/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2012                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/*! \file trimesh_base.cpp
\ingroup code_sample

\brief the minimal example of using the lib

This file contain a minimal example of the library, showing how to load a mesh and how to compute per vertex normals on it.

*/

#include<vcg/complex/complex.h>
#include<wrap/io_trimesh/import_off.h>
#include<wrap/io_trimesh/export_off.h>
#include<vcg/complex/algorithms/update/topology.h>
#include<vcg/complex/algorithms/update/normal.h>

class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public vcg::UsedTypes<vcg::Use<MyVertex>   ::AsVertexType,
                                           vcg::Use<MyEdge>     ::AsEdgeType,
                                           vcg::Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};
class MyFace    : public vcg::Face<   MyUsedTypes, vcg::face::FFAdj,  vcg::face::VertexRef, vcg::face::BitFlags > {};
class MyEdge    : public vcg::Edge<   MyUsedTypes> {};

class MyMesh    : public vcg::tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace> , std::vector<MyEdge>  > {};

using namespace vcg;

int main( int argc, char **argv )
{
  if(argc<2)
  {
    printf("Usage trimesh_base <meshfilename.off>\n");
    return -1;
  }

  MyMesh m;
  printf( "Mesh has %i vert and %i faces\n", m.VN(), m.FN() );

  tri::Allocator<MyMesh>::AddVertex(m,Point3f(1.0f,0.0f,0.0f));
  tri::Allocator<MyMesh>::AddVertex(m,Point3f(0.0f,1.0f,0.0f));
  tri::Allocator<MyMesh>::AddVertex(m,Point3f(0.0f,0.0f,1.0f));
  tri::Allocator<MyMesh>::AddVertex(m,Point3f(1.0f,1.0f,1.0f));

  printf( "Mesh has %i vert and %i faces\n", m.VN(), m.FN() );

  tri::Allocator<MyMesh>::AddFace(m,0,2,1);
  tri::Allocator<MyMesh>::AddFace(m,0,1,3);
  tri::Allocator<MyMesh>::AddFace(m,1,2,3);
  tri::Allocator<MyMesh>::AddFace(m,2,0,3);

  printf( "Mesh has %i vert and %i faces\n", m.VN(), m.FN() );

  int faceIndex = 3;
  MyFace & f = m.face[faceIndex];
  MyMesh::VertexPointer v0 = f.V(0);
  MyMesh::VertexPointer v1 = f.V(1);
  MyMesh::VertexPointer v2 = f.V(2);
  int i0 =  tri::Index(m,v0);
  int i1 =  tri::Index(m,v1);
  int i2 =  tri::Index(m,v2);

  tri::Allocator<MyMesh>::PointerUpdater<MyMesh::VertexPointer> pu;

  MyMesh::VertexIterator vi = tri::Allocator<MyMesh>::AddVertices(m,1,pu);
  pu.Update(v0);
  pu.Update(v1);
  pu.Update(v2);

  vi->P()=(v0->P()+v1->P()+v2->P())/3.0;

  tri::Allocator<MyMesh>::DeleteFace(m,f);

  tri::Allocator<MyMesh>::AddFace(m, v0, v1, &*vi);
  tri::Allocator<MyMesh>::AddFace(m, v1, v2, &*vi);
  tri::Allocator<MyMesh>::AddFace(m, v2, v0, &*vi);

  printf("m.face.size() = %i m.FN()=%i \n",m.face.size(), m.FN());
  tri::Allocator<MyMesh>::CompactFaceVector(m);
  printf("m.face.size() = %i m.FN()=%i \n",m.face.size(), m.FN());

  printf( "Mesh has %i vert and %i faces\n", m.VN(), m.FN() );

  tri::io::ExporterOFF<MyMesh>::Save(m,"tetra.off");
  return 0;
}
